@@ -38,11 +38,15 @@ if (readstata13:::dir.exists13("data"))
   unlink("data", recursive = TRUE)
 dir.create("data")
 
-dd <- mtcars
+dduni <- dd <- mtcars
+dduni$char_latin <- dd$char_latin <- sapply(1:nrow(dd), function(x) paste0(sample(c(letters, LETTERS), 9), collapse = ""))
+dduni$char_unicode <- rep(c("5ü省謹陵3_滑",  "۱碌櫓۹ۘ戀"), 16)
 
 save.dta13(dd, "data/dta_15mp.dta", version = "15mp")
-save.dta13(dd, "data/dta_119.dta", version = 119)
-save.dta13(dd, "data/dta_118.dta", version = 118)
+save.dta13(dduni, "data/dta_121.dta", version = 121)
+save.dta13(dduni, "data/dta_120.dta", version = 120)
+save.dta13(dduni, "data/dta_119.dta", version = 119)
+save.dta13(dduni, "data/dta_118.dta", version = 118)
 save.dta13(dd, "data/dta_117.dta", version = 117)
 save.dta13(dd, "data/dta_115.dta", version = 115)
 save.dta13(dd, "data/dta_114.dta", version = 114)
@@ -59,6 +63,8 @@ save.dta13(dd, "data/dta_103.dta", version = 103)
 save.dta13(dd, "data/dta_102.dta", version = 102)
 
 dd15mp<- read.dta13("data/dta_15mp.dta")
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
 dd119 <- read.dta13("data/dta_119.dta")
 dd118 <- read.dta13("data/dta_118.dta")
 dd117 <- read.dta13("data/dta_117.dta")
@@ -81,8 +87,10 @@ unlink("data", recursive = TRUE)
 
 test_that("version", {
   expect_true(datacompare(dd, dd15mp))
-  expect_true(datacompare(dd, dd119))
-  expect_true(datacompare(dd, dd118))
+  expect_true(datacompare(dduni, dd121))
+  expect_true(datacompare(dduni, dd120))
+  expect_true(datacompare(dduni, dd119))
+  expect_true(datacompare(dduni, dd118))
   expect_true(datacompare(dd, dd117))
   expect_true(datacompare(dd, dd115))
   expect_true(datacompare(dd, dd114))
@@ -109,6 +117,8 @@ dir.create("data")
 
 dd <- mtcars
 
+save.dta13(dd, "data/dta_121.dta", version = 121, compress = TRUE)
+save.dta13(dd, "data/dta_120.dta", version = 120, compress = TRUE)
 save.dta13(dd, "data/dta_119.dta", version = 119, compress = TRUE)
 save.dta13(dd, "data/dta_118.dta", version = 118, compress = TRUE)
 save.dta13(dd, "data/dta_117.dta", version = 117, compress = TRUE)
@@ -126,6 +136,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104, compress = TRUE)
 save.dta13(dd, "data/dta_103.dta", version = 103, compress = TRUE)
 save.dta13(dd, "data/dta_102.dta", version = 102, compress = TRUE)
 
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
 dd119 <- read.dta13("data/dta_119.dta")
 dd118 <- read.dta13("data/dta_118.dta")
 dd117 <- read.dta13("data/dta_117.dta")
@@ -147,6 +159,8 @@ dd102 <- read.dta13("data/dta_102.dta")
 unlink("data", recursive = TRUE)
 
 test_that("compress", {
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -176,6 +190,8 @@ dir.create("data")
 dd <- mtcars
 dd$am <- factor(x = dd$am, levels = c(0,1), labels = c("auto", "man"))
 
+save.dta13(dd, "data/dta_121.dta", version = 121, convert.factors = TRUE)
+save.dta13(dd, "data/dta_120.dta", version = 120, convert.factors = TRUE)
 save.dta13(dd, "data/dta_119.dta", version = 119, convert.factors = TRUE)
 save.dta13(dd, "data/dta_118.dta", version = 118, convert.factors = TRUE)
 save.dta13(dd, "data/dta_117.dta", version = 117, convert.factors = TRUE)
@@ -193,6 +209,8 @@ save.dta13(dd, "data/dta_107.dta", version = 107, convert.factors = TRUE)
 # save.dta13(dd, "data/dta_103.dta", version = 103, convert.factors = TRUE)
 # save.dta13(dd, "data/dta_102.dta", version = 102, convert.factors = TRUE)
 
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
 dd119 <- read.dta13("data/dta_119.dta")
 dd118 <- read.dta13("data/dta_118.dta")
 dd117 <- read.dta13("data/dta_117.dta")
@@ -214,6 +232,8 @@ dd107 <- read.dta13("data/dta_107.dta")
 unlink("data", recursive = TRUE)
 
 test_that("convert.factors TRUE", {
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -243,6 +263,8 @@ dir.create("data")
 dd <- mtcars
 dd$am <- factor(x = dd$am, levels = c(0,1), labels = c("auto", "man"))
 
+save.dta13(dd, "data/dta_121.dta", version = 121, convert.factors = FALSE)
+save.dta13(dd, "data/dta_120.dta", version = 120, convert.factors = FALSE)
 save.dta13(dd, "data/dta_119.dta", version = 119, convert.factors = FALSE)
 save.dta13(dd, "data/dta_118.dta", version = 118, convert.factors = FALSE)
 save.dta13(dd, "data/dta_117.dta", version = 117, convert.factors = FALSE)
@@ -260,6 +282,8 @@ save.dta13(dd, "data/dta_107.dta", version = 107, convert.factors = FALSE)
 # save.dta13(dd, "data/dta_103.dta", version = 103, convert.factors = FALSE)
 # save.dta13(dd, "data/dta_102.dta", version = 102, convert.factors = FALSE)
 
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
 dd119 <- read.dta13("data/dta_119.dta")
 dd118 <- read.dta13("data/dta_118.dta")
 dd117 <- read.dta13("data/dta_117.dta")
@@ -285,6 +309,8 @@ dd$am <- dd$am + 1
 unlink("data", recursive = TRUE)
 
 test_that("convert.factors TRUE", {
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -312,6 +338,8 @@ dir.create("data")
 
 dd <- mtcars
 
+save.dta13(dd, "data/dta_121.dta", version = 121, add.rownames = TRUE)
+save.dta13(dd, "data/dta_120.dta", version = 120, add.rownames = TRUE)
 save.dta13(dd, "data/dta_119.dta", version = 119, add.rownames = TRUE)
 save.dta13(dd, "data/dta_118.dta", version = 118, add.rownames = TRUE)
 save.dta13(dd, "data/dta_117.dta", version = 117, add.rownames = TRUE)
@@ -329,6 +357,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104, add.rownames = TRUE)
 save.dta13(dd, "data/dta_103.dta", version = 103, add.rownames = TRUE)
 save.dta13(dd, "data/dta_102.dta", version = 102, add.rownames = TRUE)
 
+dd121 <- read.dta13("data/dta_121.dta", add.rownames = TRUE)
+dd120 <- read.dta13("data/dta_120.dta", add.rownames = TRUE)
 dd119 <- read.dta13("data/dta_119.dta", add.rownames = TRUE)
 dd118 <- read.dta13("data/dta_118.dta", add.rownames = TRUE)
 dd117 <- read.dta13("data/dta_117.dta", add.rownames = TRUE)
@@ -351,6 +381,8 @@ unlink("data", recursive = TRUE)
 
 test_that("add.rownames TRUE", {
   # Check that rownames are identical
+  expect_true(identical(rownames(dd), rownames(dd121)))
+  expect_true(identical(rownames(dd), rownames(dd120)))
   expect_true(identical(rownames(dd), rownames(dd119)))
   expect_true(identical(rownames(dd), rownames(dd118)))
   expect_true(identical(rownames(dd), rownames(dd117)))
@@ -369,6 +401,8 @@ test_that("add.rownames TRUE", {
   expect_true(identical(rownames(dd), rownames(dd102)))
 
   # Check that data is identical
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -399,6 +433,8 @@ dir.create("data")
 
 dd <- mtcars
 
+save.dta13(dd, "data/dta_121.dta", version = 121, data.label = dl)
+save.dta13(dd, "data/dta_120.dta", version = 120, data.label = dl)
 save.dta13(dd, "data/dta_119.dta", version = 119, data.label = dl)
 save.dta13(dd, "data/dta_118.dta", version = 118, data.label = dl)
 save.dta13(dd, "data/dta_117.dta", version = 117, data.label = dl)
@@ -416,6 +452,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104, data.label = dl)
 save.dta13(dd, "data/dta_103.dta", version = 103, data.label = dl)
 # save.dta13(dd, "data/dta_102.dta", version = 102, data.label = dl) # no data label
 
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
 dd119 <- read.dta13("data/dta_119.dta")
 dd118 <- read.dta13("data/dta_118.dta")
 dd117 <- read.dta13("data/dta_117.dta")
@@ -437,6 +475,8 @@ unlink("data", recursive = TRUE)
 
 test_that("data label", {
   # Check that rownames are identical
+  expect_equal(dl, attr(dd121, "datalabel"))
+  expect_equal(dl, attr(dd120, "datalabel"))
   expect_equal(dl, attr(dd119, "datalabel"))
   expect_equal(dl, attr(dd118, "datalabel"))
   expect_equal(dl, attr(dd117, "datalabel"))
@@ -502,6 +542,8 @@ dd <- data.frame(td = as.Date(td),
                  tm = as.Date(tm),
                  tq = as.Date(tq))
 
+save.dta13(dd, "data/dta_121.dta", version = 121, convert.dates = TRUE)
+save.dta13(dd, "data/dta_120.dta", version = 120, convert.dates = TRUE)
 save.dta13(dd, "data/dta_119.dta", version = 119, convert.dates = TRUE)
 save.dta13(dd, "data/dta_118.dta", version = 118, convert.dates = TRUE)
 save.dta13(dd, "data/dta_117.dta", version = 117, convert.dates = TRUE)
@@ -519,6 +561,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104, convert.dates = TRUE)
 save.dta13(dd, "data/dta_103.dta", version = 103, convert.dates = TRUE)
 save.dta13(dd, "data/dta_102.dta", version = 102, convert.dates = TRUE)
 
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
 dd119 <- read.dta13("data/dta_119.dta")
 dd118 <- read.dta13("data/dta_118.dta")
 dd117 <- read.dta13("data/dta_117.dta")
@@ -540,6 +584,8 @@ unlink("data", recursive = TRUE)
 
 test_that("convert.dates TRUE", {
   # Check that rownames are identical
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -571,6 +617,8 @@ dd <- data.frame( dat = c(paste(replicate(2046, "a"), collapse = ""),
                           paste(replicate(2046, "b"), collapse = "")),
                   stringsAsFactors = FALSE)
 
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
 save.dta13(dd, "data/dta_119.dta", version = 119)
 save.dta13(dd, "data/dta_118.dta", version = 118)
 save.dta13(dd, "data/dta_117.dta", version = 117)
@@ -588,6 +636,8 @@ save.dta13(dd, "data/dta_117.dta", version = 117)
 # save.dta13(dd, "data/dta_103.dta", version = 103)
 # save.dta13(dd, "data/dta_102.dta", version = 102)
 
+dd121 <- read.dta13("data/dta_121.dta", replace.strl = TRUE)
+dd120 <- read.dta13("data/dta_120.dta", replace.strl = TRUE)
 dd119 <- read.dta13("data/dta_119.dta", replace.strl = TRUE)
 dd118 <- read.dta13("data/dta_118.dta", replace.strl = TRUE)
 dd117 <- read.dta13("data/dta_117.dta", replace.strl = TRUE)
@@ -609,6 +659,8 @@ unlink("data", recursive = TRUE)
 
 test_that("replace.strl TRUE", {
   # Check that rownames are identical
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -636,6 +688,8 @@ dir.create("data")
 
 dd <- data.frame(x.1 = 1)
 
+save.dta13(dd, "data/dta_121.dta", version = 121, convert.underscore = TRUE)
+save.dta13(dd, "data/dta_120.dta", version = 120, convert.underscore = TRUE)
 save.dta13(dd, "data/dta_119.dta", version = 119, convert.underscore = TRUE)
 save.dta13(dd, "data/dta_118.dta", version = 118, convert.underscore = TRUE)
 save.dta13(dd, "data/dta_117.dta", version = 117, convert.underscore = TRUE)
@@ -653,6 +707,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104, convert.underscore = TRUE)
 save.dta13(dd, "data/dta_103.dta", version = 103, convert.underscore = TRUE)
 save.dta13(dd, "data/dta_102.dta", version = 102, convert.underscore = TRUE)
 
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
 dd119 <- read.dta13("data/dta_119.dta")
 dd118 <- read.dta13("data/dta_118.dta")
 dd117 <- read.dta13("data/dta_117.dta")
@@ -676,6 +732,8 @@ names(dd) <- "x_1"
 
 test_that("convert.underscore TRUE", {
   # check numerics
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -693,6 +751,8 @@ test_that("convert.underscore TRUE", {
   expect_true(datacompare(dd, dd103))
   expect_true(datacompare(dd, dd102))
   # check names
+  expect_true(namescompare(dd, dd121))
+  expect_true(namescompare(dd, dd120))
   expect_true(namescompare(dd, dd119))
   expect_true(namescompare(dd, dd118))
   expect_true(namescompare(dd, dd117))
@@ -720,6 +780,8 @@ dir.create("data")
 
 dd <- mtcars
 
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
 save.dta13(dd, "data/dta_119.dta", version = 119)
 save.dta13(dd, "data/dta_118.dta", version = 118)
 save.dta13(dd, "data/dta_117.dta", version = 117)
@@ -737,6 +799,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104)
 save.dta13(dd, "data/dta_103.dta", version = 103)
 save.dta13(dd, "data/dta_102.dta", version = 102)
 
+dd121 <- read.dta13("data/dta_121.dta", select.rows = 5)
+dd120 <- read.dta13("data/dta_120.dta", select.rows = 5)
 dd119 <- read.dta13("data/dta_119.dta", select.rows = 5)
 dd118 <- read.dta13("data/dta_118.dta", select.rows = 5)
 dd117 <- read.dta13("data/dta_117.dta", select.rows = 5)
@@ -760,6 +824,8 @@ dd <- dd[1:5,]
 
 test_that("select.rows = 5", {
   # check numerics
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -784,6 +850,8 @@ dir.create("data")
 
 dd <- mtcars
 
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
 save.dta13(dd, "data/dta_119.dta", version = 119)
 save.dta13(dd, "data/dta_118.dta", version = 118)
 save.dta13(dd, "data/dta_117.dta", version = 117)
@@ -801,6 +869,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104)
 save.dta13(dd, "data/dta_103.dta", version = 103)
 save.dta13(dd, "data/dta_102.dta", version = 102)
 
+dd121 <- read.dta13("data/dta_121.dta", select.rows = c(5,10))
+dd120 <- read.dta13("data/dta_120.dta", select.rows = c(5,10))
 dd119 <- read.dta13("data/dta_119.dta", select.rows = c(5,10))
 dd118 <- read.dta13("data/dta_118.dta", select.rows = c(5,10))
 dd117 <- read.dta13("data/dta_117.dta", select.rows = c(5,10))
@@ -824,6 +894,8 @@ dd <- dd[5:10,]
 
 test_that("select.rows = c(5,10)", {
   # check numerics
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -852,6 +924,8 @@ dir.create("data")
 
 dd <- mtcars
 
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
 save.dta13(dd, "data/dta_119.dta", version = 119)
 save.dta13(dd, "data/dta_118.dta", version = 118)
 save.dta13(dd, "data/dta_117.dta", version = 117)
@@ -869,6 +943,8 @@ save.dta13(dd, "data/dta_104.dta", version = 104)
 save.dta13(dd, "data/dta_103.dta", version = 103)
 save.dta13(dd, "data/dta_102.dta", version = 102)
 
+dd121 <- read.dta13("data/dta_121.dta", select.cols = c("disp", "drat"))
+dd120 <- read.dta13("data/dta_120.dta", select.cols = c("disp", "drat"))
 dd119 <- read.dta13("data/dta_119.dta", select.cols = c("disp", "drat"))
 dd118 <- read.dta13("data/dta_118.dta", select.cols = c("disp", "drat"))
 dd117 <- read.dta13("data/dta_117.dta", select.cols = c("disp", "drat"))
@@ -892,6 +968,8 @@ dd <- dd[,c("disp", "drat")]
 
 test_that("select.cols = c('disp', 'drat')", {
   # check numerics
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
   expect_true(datacompare(dd, dd119))
   expect_true(datacompare(dd, dd118))
   expect_true(datacompare(dd, dd117))
@@ -910,6 +988,78 @@ test_that("select.cols = c('disp', 'drat')", {
   expect_true(datacompare(dd, dd102))
 })
 
+if (readstata13:::dir.exists13("data"))
+  unlink("data", recursive = TRUE)
+dir.create("data")
+
+dd <- mtcars
+
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
+save.dta13(dd, "data/dta_119.dta", version = 119)
+save.dta13(dd, "data/dta_118.dta", version = 118)
+save.dta13(dd, "data/dta_117.dta", version = 117)
+save.dta13(dd, "data/dta_115.dta", version = 115)
+save.dta13(dd, "data/dta_114.dta", version = 114)
+save.dta13(dd, "data/dta_113.dta", version = 113)
+save.dta13(dd, "data/dta_112.dta", version = 112)
+save.dta13(dd, "data/dta_111.dta", version = 111)
+save.dta13(dd, "data/dta_110.dta", version = 110)
+save.dta13(dd, "data/dta_108.dta", version = 108)
+save.dta13(dd, "data/dta_107.dta", version = 107)
+save.dta13(dd, "data/dta_106.dta", version = 106)
+save.dta13(dd, "data/dta_105.dta", version = 105)
+save.dta13(dd, "data/dta_104.dta", version = 104)
+save.dta13(dd, "data/dta_103.dta", version = 103)
+save.dta13(dd, "data/dta_102.dta", version = 102)
+
+dd121 <- read.dta13("data/dta_121.dta", select.cols = c(3, 5))
+dd120 <- read.dta13("data/dta_120.dta", select.cols = c(3, 5))
+dd119 <- read.dta13("data/dta_119.dta", select.cols = c(3, 5))
+dd118 <- read.dta13("data/dta_118.dta", select.cols = c(3, 5))
+dd117 <- read.dta13("data/dta_117.dta", select.cols = c(3, 5))
+dd115 <- read.dta13("data/dta_115.dta", select.cols = c(3, 5))
+dd114 <- read.dta13("data/dta_114.dta", select.cols = c(3, 5))
+dd113 <- read.dta13("data/dta_113.dta", select.cols = c(3, 5))
+dd112 <- read.dta13("data/dta_112.dta", select.cols = c(3, 5))
+dd111 <- read.dta13("data/dta_111.dta", select.cols = c(3, 5))
+dd110 <- read.dta13("data/dta_110.dta", select.cols = c(3, 5))
+dd108 <- read.dta13("data/dta_108.dta", select.cols = c(3, 5))
+dd107 <- read.dta13("data/dta_107.dta", select.cols = c(3, 5))
+dd106 <- read.dta13("data/dta_106.dta", select.cols = c(3, 5))
+dd105 <- read.dta13("data/dta_105.dta", select.cols = c(3, 5))
+dd104 <- read.dta13("data/dta_104.dta", select.cols = c(3, 5))
+dd103 <- read.dta13("data/dta_103.dta", select.cols = c(3, 5))
+dd102 <- read.dta13("data/dta_102.dta", select.cols = c(3, 5))
+
+unlink("data", recursive = TRUE)
+
+dd <- dd[,c("disp", "drat")]
+
+test_that("select.cols = c('disp', 'drat')", {
+  # check numerics
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
+  expect_true(datacompare(dd, dd119))
+  expect_true(datacompare(dd, dd118))
+  expect_true(datacompare(dd, dd117))
+  expect_true(datacompare(dd, dd115))
+  expect_true(datacompare(dd, dd114))
+  expect_true(datacompare(dd, dd113))
+  expect_true(datacompare(dd, dd112))
+  expect_true(datacompare(dd, dd111))
+  expect_true(datacompare(dd, dd110))
+  expect_true(datacompare(dd, dd108))
+  expect_true(datacompare(dd, dd107))
+  expect_true(datacompare(dd, dd106))
+  expect_true(datacompare(dd, dd105))
+  expect_true(datacompare(dd, dd104))
+  expect_true(datacompare(dd, dd103))
+  expect_true(datacompare(dd, dd102))
+})
+
+
+
 # rm(list = files)
 
 
@@ -944,6 +1094,8 @@ ef <- list(
 
 attr(dd, "expansion.fields") <- ef
 
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
 save.dta13(dd, "data/dta_119.dta", version = 119)
 save.dta13(dd, "data/dta_118.dta", version = 118)
 save.dta13(dd, "data/dta_117.dta", version = 117)
@@ -961,6 +1113,8 @@ save.dta13(dd, "data/dta_105.dta", version = 105)
 # save.dta13(dd, "data/dta_103.dta", version = 103)
 # save.dta13(dd, "data/dta_102.dta", version = 102)
 
+dd121 <- attr(read.dta13("data/dta_121.dta"), "expansion.fields")
+dd120 <- attr(read.dta13("data/dta_120.dta"), "expansion.fields")
 dd119 <- attr(read.dta13("data/dta_119.dta"), "expansion.fields")
 dd118 <- attr(read.dta13("data/dta_118.dta"), "expansion.fields")
 dd117 <- attr(read.dta13("data/dta_117.dta"), "expansion.fields")
@@ -982,6 +1136,8 @@ unlink("data", recursive = TRUE)
 
 test_that("expansinon.fields", {
   # check numerics
+  expect_equal(ef, dd121)
+  expect_equal(ef, dd120)
   expect_equal(ef, dd119)
   expect_equal(ef, dd118)
   expect_equal(ef, dd117)
@@ -1000,7 +1156,7 @@ test_that("expansinon.fields", {
   # expect_equal(ef, dd102)
 })
 
-#### save and read varlabels  #### 
+#### save and read varlabels  ####
 
 if (readstata13:::dir.exists13("data")) {
   unlink("data", recursive = TRUE)
@@ -1011,8 +1167,8 @@ dd <- mtcars
 varlabeldd <- LETTERS[seq_len(ncol(dd))]
 varlabel(dd) <- varlabeldd
 
-version_list <- c(102,103,104,105,106,107,108,110,
-                  111,112,113,114,115,117,118,119)
+version_list <- c(102,103,104,105,106,107,108,110,111,
+                  112,113,114,115,117,118,119,120,121)
 
 # write variable label attribute
 for(v in version_list) {
@@ -1020,20 +1176,120 @@ for(v in version_list) {
 }
 
 # read variable label attribute
-varlabeldd_read <- lapply(version_list, 
+varlabeldd_read <- lapply(version_list,
                           function(v) {
-                            attr(read.dta13(paste0("data/dta_", v, ".dta")), 
+                            attr(read.dta13(paste0("data/dta_", v, ".dta")),
                                  "var.labels")
                           })
 names(varlabeldd_read) <- as.character(version_list)
-  
+
 unlink("data", recursive = TRUE)
 
 test_that("save and read varlabels", {
-  
+
   for(v in as.character(version_list)) {
     expect_equal(varlabeldd, varlabeldd_read[[v]])
   }
-  
+
 })
 
+
+#### differentiating "NA" and NA_character works  ####
+
+if (readstata13:::dir.exists13("data"))
+  unlink("data", recursive = TRUE)
+dir.create("data")
+
+dd <- data.frame(x1 = c("NA", NA_character_))
+exp <- data.frame(x1 = c("NA", ""))
+
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
+save.dta13(dd, "data/dta_119.dta", version = 119)
+save.dta13(dd, "data/dta_118.dta", version = 118)
+save.dta13(dd, "data/dta_117.dta", version = 117)
+save.dta13(dd, "data/dta_115.dta", version = 115)
+save.dta13(dd, "data/dta_114.dta", version = 114)
+save.dta13(dd, "data/dta_113.dta", version = 113)
+save.dta13(dd, "data/dta_112.dta", version = 112)
+save.dta13(dd, "data/dta_111.dta", version = 111)
+save.dta13(dd, "data/dta_110.dta", version = 110)
+save.dta13(dd, "data/dta_108.dta", version = 108)
+save.dta13(dd, "data/dta_107.dta", version = 107)
+save.dta13(dd, "data/dta_106.dta", version = 106)
+save.dta13(dd, "data/dta_105.dta", version = 105)
+save.dta13(dd, "data/dta_104.dta", version = 104)
+save.dta13(dd, "data/dta_103.dta", version = 103)
+save.dta13(dd, "data/dta_102.dta", version = 102)
+
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
+dd119 <- read.dta13("data/dta_119.dta")
+dd118 <- read.dta13("data/dta_118.dta")
+dd117 <- read.dta13("data/dta_117.dta")
+dd115 <- read.dta13("data/dta_115.dta")
+dd114 <- read.dta13("data/dta_114.dta")
+dd113 <- read.dta13("data/dta_113.dta")
+dd112 <- read.dta13("data/dta_112.dta")
+dd111 <- read.dta13("data/dta_111.dta")
+dd110 <- read.dta13("data/dta_110.dta")
+dd108 <- read.dta13("data/dta_108.dta")
+dd107 <- read.dta13("data/dta_107.dta")
+dd106 <- read.dta13("data/dta_106.dta")
+dd105 <- read.dta13("data/dta_105.dta")
+dd104 <- read.dta13("data/dta_104.dta")
+dd103 <- read.dta13("data/dta_103.dta")
+dd102 <- read.dta13("data/dta_102.dta")
+
+test_that("NA character works", {
+  expect_true(datacompare(exp, dd121))
+  expect_true(datacompare(exp, dd120))
+  expect_true(datacompare(exp, dd119))
+  expect_true(datacompare(exp, dd118))
+  expect_true(datacompare(exp, dd117))
+  expect_true(datacompare(exp, dd115))
+  expect_true(datacompare(exp, dd114))
+  expect_true(datacompare(exp, dd113))
+  expect_true(datacompare(exp, dd112))
+  expect_true(datacompare(exp, dd111))
+  expect_true(datacompare(exp, dd110))
+  expect_true(datacompare(exp, dd108))
+  expect_true(datacompare(exp, dd107))
+  expect_true(datacompare(exp, dd106))
+  expect_true(datacompare(exp, dd105))
+  expect_true(datacompare(exp, dd104))
+  expect_true(datacompare(exp, dd103))
+  expect_true(datacompare(exp, dd102))
+})
+
+# the same with strls
+if (readstata13:::dir.exists13("data"))
+  unlink("data", recursive = TRUE)
+dir.create("data")
+
+# strLs can be of length any length up to 2 billion characters. Starting with
+# 2046 a string is handled as a strL
+dd <- data.frame( dat = c(paste(replicate(2046, "a"), collapse = ""),
+                          paste(replicate(2046, "b"), collapse = ""),
+                          "NA", NA_character_),
+                  stringsAsFactors = FALSE)
+
+save.dta13(dd, "data/dta_121.dta", version = 121)
+save.dta13(dd, "data/dta_120.dta", version = 120)
+save.dta13(dd, "data/dta_119.dta", version = 119)
+save.dta13(dd, "data/dta_118.dta", version = 118)
+save.dta13(dd, "data/dta_117.dta", version = 117)
+
+dd121 <- read.dta13("data/dta_121.dta")
+dd120 <- read.dta13("data/dta_120.dta")
+dd119 <- read.dta13("data/dta_119.dta")
+dd118 <- read.dta13("data/dta_118.dta")
+dd117 <- read.dta13("data/dta_117.dta")
+
+test_that("NA character works", {
+  expect_true(datacompare(dd, dd121))
+  expect_true(datacompare(dd, dd120))
+  expect_true(datacompare(dd, dd119))
+  expect_true(datacompare(dd, dd118))
+  expect_true(datacompare(dd, dd117))
+})
